<?php

declare(strict_types=1);

namespace Drupal\mercury_editor_task;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;

/**
 * The Mercury Editor Task form display builder service.
 */
class MercuryEditorTaskFormDisplayBuilder implements MercuryEditorTaskFormDisplayBuilderInterface {

  /**
   * Constructs a MercuryEditorTaskFormDisplayBuilder object.
   */
  public function __construct(
    protected ConfigFactoryInterface $configFactory,
    protected EntityDisplayRepositoryInterface $entityDisplayRepository,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function update(bool $force = FALSE): bool {
    $updated = FALSE;

    $bundles = $this->configFactory
      ->get('mercury_editor.settings')
      ->get('bundles.node') ?? [];

    foreach ($bundles as $bundle) {
      $default_display = $this->entityDisplayRepository->getFormDisplay('node', $bundle, 'default');
      $mercury_editor_display = $this->entityDisplayRepository->getFormDisplay('node', $bundle, 'mercury_editor');
      if (!$mercury_editor_display->isNew() && !$force) {
        continue;
      }

      $displayed_components = $this->configFactory
        ->get('mercury_editor_task.settings')
        ->get('components');

      // If components are empty, delete the display.
      if (empty($displayed_components)) {
        $mercury_editor_display->delete();
        continue;
      }

      // Append layout paragraphs to components.
      $has_layout_paragraphs = FALSE;
      $components = $default_display->getComponents();
      foreach ($components as $component_name => $component) {
        if (isset($component['type'])
          && $component['type'] === 'layout_paragraphs') {
          $has_layout_paragraphs = TRUE;
          $displayed_components[] = $component_name;
        }
      }
      // If the default display, does not use layout paragraphs,
      // we should not create or update the dedicated 'mercury_editor' display.
      if (!$has_layout_paragraphs) {
        continue;
      }

      // Append field group children to displayed components.
      $field_group = $default_display->getThirdPartySettings('field_group');
      foreach ($field_group as $group_name => $group) {
        if (in_array($group_name, $displayed_components)) {
          $displayed_components = array_merge($displayed_components, $group['children']);
        }
      }

      $displayed_components = array_unique($displayed_components);

      // Copy the default display's properties, instead of duplicating the display to
      // ensure existing uuids are not overwritten.
      $display_properties = ['langcode', 'dependencies', 'third_party_settings', 'content', 'hidden'];
      foreach ($display_properties as $display_property) {
        $mercury_editor_display->set(
          $display_property,
          $default_display->get($display_property)
        );
      }

      // Make sure an existing display is always enabled.
      $mercury_editor_display->setStatus(TRUE);

      // Remove components.
      $components = $mercury_editor_display->getComponents();
      foreach ($components as $component_name => $component) {
        if (!in_array($component_name, $displayed_components)) {
          $mercury_editor_display->removeComponent($component_name);
        }
      }

      // Update field groups.
      $field_group = $mercury_editor_display->getThirdPartySettings('field_group');
      if ($field_group) {
        foreach ($field_group as $group_name => $group) {
          $group['children'] = array_intersect($group['children'], $displayed_components);
          if (empty($group['children'])) {
            $mercury_editor_display->unsetThirdPartySetting('field_group', $group_name);
          }
          else {
            $mercury_editor_display->setThirdPartySetting('field_group', $group_name, $group);
          }
        }
      }

      // Save the display.
      $mercury_editor_display->save();

      // Set updated status.
      $updated = TRUE;
    }

    return $updated;
  }

}
